#!perl

use IO::File;

sub add_missing_declaration{
	
	my $fho = shift;
	
	$fho->print("public:\n");
	$fho->print("\tvirtual void soap_default(struct soap*);\n");
	$fho->print("\tvirtual void soap_serialize(struct soap*) const;\n");
	$fho->print("\tvirtual void soap_mark(struct soap*) const;\n");
	$fho->print("\tvirtual int soap_put(struct soap*, const char*, const char*) const;\n");
	$fho->print("\tvirtual int soap_out(struct soap*, const char*, int, const char*) const;\n");
	$fho->print("\tvirtual void *soap_get(struct soap*, const char*, const char*);\n");
	$fho->print("\tvirtual void *soap_in(struct soap*, const char*, const char*);\n");
}

sub fix_fault_detail {

	my $STATUS_NORMAL      = 0;
	my $STATUS_COMMENT     = 1;
	my $STATUS_CLASS       = 2;
	my $STATUS_CLASS_OPEN  = 3;
	my $STATUS_DONE	 	   = 4;
	
	my $fault_detail_class = shift;
	my $fname 			   = shift;
	
	my $fname_tmp = $fname . ".tmp";
	my $fhi = new IO::File $fname , O_RDONLY;
	my $fho = new IO::File $fname_tmp, O_WRONLY | O_CREAT | O_TRUNC;
	if ((!defined $fhi) || (!defined $fho)) {
		die "Cannot Open File : $!";
    }
    my $status   = $STATUS_NORMAL;
    my $need_fix = 0;
    my $pattern  = "";
	while (<$fhi>){
		my $content = $_;
		if($status == $STATUS_NORMAL) {
			# Check if the line correspond to /* $fault_detail_class: */
			$pattern = "^/(\\*)(\\s)*$fault_detail_class:(\\s)*(\\*)/";
			if( $content =~ m/$pattern/){
				#print "Detail Found : $content\n";
				$status = $STATUS_COMMENT;
			}
		} elsif($status == $STATUS_COMMENT) {
			# Line should be class SOAP_CMAC _$fault_detail_class
			$pattern = "^class SOAP_CMAC _$fault_detail_class";
			if( $content =~ m/$pattern/){
				#print "Class Found : $content\n";
				$status = $STATUS_CLASS;
			}
		} elsif($status == $STATUS_CLASS) {
			# Next Line Should be {
			if( $content =~ m/^{/){
				#print "Class Open Found : $content\n";
				$status = $STATUS_CLASS_OPEN;
				$need_fix = 1;
			}
		} elsif($status == $STATUS_CLASS_OPEN) {
			# Copy Any Line until }; checking if the declarations are missing
			$pattern = "^(\\s)*virtual void (\\*)soap_in\\(struct soap(\\*), const char(\\*), const char(\\*)\\);(\\s)*";
			if( $content =~ m/$pattern/){
				print "Don't need fix !\n";
				$need_fix = 0;
			}
			if( $content =~ m/^};/){
				# print "Class Close Found : $content\n";
				$status = $STATUS_DONE;
				# Modify the Class, if needed
				if($need_fix == 1){
					print "Need Fix !\n";
					add_missing_declaration($fho);
				}
			}
		}
		$fho->print($content);
		
	}
	
	$fhi->close();
	$fho->close();
	
	rename($fname_tmp, $fname) || die "Cannot rename file: $!";
}

fix_fault_detail(shift,shift);
